package setup;

import core.Position;
import core.LufBoard;
import core.TileType;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashSet;
import java.util.Random;

/**
 A simple board factory which places all tiles with equal probability.

 @author reidl
 */
public class RandomBoardBuilder extends ProceduralBoardBuilder {

    private Random rnd = new Random(System.currentTimeMillis());

    public RandomBoardBuilder setSeed(long seed) {
        rnd.setSeed(seed);
        return this; // For chaining
    }

    @Override
    protected LufBoard buildInternal(LufBoard res) {
        TileType[] tiles = TileType.values();
        TileType[][] tileMatrix = new TileType[width][height];
        int len = tiles.length;
        for (int x = 0; x < width; x++) {
            for (int y = 0; y < height; y++) {
                tileMatrix[x][y] = tiles[rnd.nextInt(len)];
            }
        }

        int minDim = Math.min(width,height);
        int numFields = width*height;
        int numFixedTiles = rnd.nextInt(minDim / 2);

        // Distribute objectives on players
        int objPerPlayer = rnd.nextInt(numFields/2) + 1; // TODO
        ArrayList<Integer> allObjectives = new ArrayList<Integer>();
        for (int i = 0; i < objPerPlayer * numPlayers; i++) {
            allObjectives.add(i);
        }
        Collections.shuffle(allObjectives, rnd);

        ArrayList<ArrayList<Integer>> objectives = new ArrayList<ArrayList<Integer>>();
        for (int p = 0; p < numPlayers; p++) {
            ArrayList<Integer> playerObjectives = new ArrayList<Integer>();

            for (int i = p * objPerPlayer; i < (p + 1) * objPerPlayer; i++) {
                playerObjectives.add(allObjectives.get(i));
            }

            objectives.add(playerObjectives);
        }

        // Create objective positions
        ArrayList<Position> objectivePositions = new ArrayList<Position>();
        for (int i = 0; i < allObjectives.size(); i++) {
            Position pos = new Position(rnd.nextInt(width), rnd.nextInt(height));
            while (tileMatrix[pos.x][pos.y] == TileType.BLANK) {
                pos.x = rnd.nextInt(width);
                pos.y = rnd.nextInt(height);
            }
            objectivePositions.add(pos);
        }

        ArrayList<Position> playerPositions = new ArrayList<Position>();
        for (int i = 0; i < numPlayers; i++) {
            Position pos = new Position(rnd.nextInt(width), rnd.nextInt(height));
            while (tileMatrix[pos.x][pos.y] == TileType.BLANK) {
                pos.x = rnd.nextInt(width);
                pos.y = rnd.nextInt(height);
            }
            playerPositions.add(pos);
        }

        HashSet<Position> fixedTiles = new HashSet<Position>();
        fixedTiles.addAll(Arrays.asList(randomPositions(numFixedTiles)));

        // Check for bad 2x2 configuration in board and resolve by
        // making some tiles movable again
        Position topLeft = new Position(0,0);
        Position topRight = new Position(0,0);
        Position bottomLeft = new Position(0,0);
        Position bottomRight = new Position(0,0);
        for (int x = 0; x < width-1; x++) {
            topLeft.x = bottomLeft.x = x;
            topRight.x = bottomRight.x = x+1;
            for (int y = 0; y < height-1; y++) {
                topLeft.y = topRight.y = y;
                bottomLeft.y = bottomRight.y = y;
                if( fixedTiles.contains(topLeft) && fixedTiles.contains(bottomRight)) {
                    fixedTiles.remove(bottomRight);
                } 
                
                if(fixedTiles.contains(topRight) && fixedTiles.contains(bottomLeft)) {
                    fixedTiles.remove(bottomLeft);
                }
            }
        }

        res.setBoard(tileMatrix, tiles[rnd.nextInt(len)],
                objectivePositions.toArray(new Position[0]),
                playerPositions.toArray(new Position[0]),
                fixedTiles.toArray(new Position[0]));

        for (int i = 0; i < numPlayers; i++) {
            res.setPlayerObjectives(i, objectives.get(i));
        }

        return res;
    }

    private Position[] randomPositions(int numFixedTiles) {
        HashSet<Position> positions = new HashSet<Position>();
        for (int i = 0; i < numFixedTiles; i++) {
            Position pos = Position.create(rnd.nextInt(width), rnd.nextInt(height));
            positions.add(pos);
        }
        return positions.toArray(new Position[0]);
    }

    @Override
    public String toString() {
        return "RandomBoardBuilder{width=" + width +",height=" + height + ",rnd=" + rnd + '}';
    }
    
    
}
